/* - IR full -
 * This example displays a gui that shows the attached Phidgets IR reader device's details and will
 * display and tag data that is scanned by the antenna.  it also provides checkboxes to manipulate the
 * different attirbutes of the IR reader as well as the digital outputs.
 *
 * Please note that this example was designed to work with only one Phidget IR connected.
 * For an example showing how to use two Phidgets of the same time concurrently, please see the
 * Servo-multi example in the Servo Examples.
 *
 * Copyright 2015 Phidgets Inc.
 * This work is licensed under the Creative Commons Attribution 2.5 Canada License.
 * To view a copy of this license, visit http://creativecommons.org/licenses/by/2.5/ca/
 */
using Phidget22;
using Phidget22.Events;
using Phidget22.ExampleUtils;
using System;
using System.Collections.Generic;
using System.Text;
using System.Threading;
using System.Windows.Forms;

namespace IR_Example {
	public partial class Form1 : Form {
		CommandLineOpen openArgs;
		IR openCh = null;
		private IR ir; //Declare an IR object
		private ErrorEventBox errorBox;
		int repeatCount = 0;
		formCleaner cleaner;

		bool buttonPressed = false;

		int rawDataCol = 0;

		public Form1() {
			InitializeComponent();
			openArgs = new CommandLineOpen(this);
			cleaner = new formCleaner(this);
		}

		public Form1(IR ch) {
			InitializeComponent();
			openCh = ch;
			cleaner = new formCleaner(this);
		}

		//initialize our Phidgets IR reader and hook the event handlers
		private void Form1_Load(object sender, EventArgs e) {
			errorBox = new ErrorEventBox(this);
			ir = openCh ?? openArgs.makeChannel<IR>();

			ir.Attach += ir_Attach;
			ir.Detach += ir_Detach;
			ir.Error += ir_Error;

			ir.Code += ir_Code;
			ir.Learn += ir_Learn;
			ir.RawData += ir_RawData;

			try {
				ir.Open();
			} catch (PhidgetException ex) { errorBox.addMessage("Error opening device: " + ex.Message); }
		}

		//When the application is being terminated, close the Phidget.
		private void Form1_FormClosing(object sender, FormClosingEventArgs e) {

			ir.Attach -= ir_Attach;
			ir.Detach -= ir_Detach;
			ir.Error -= ir_Error;
			ir.Code -= ir_Code;
			ir.Learn -= ir_Learn;
			ir.RawData -= ir_RawData;
			ir.Close();
		}

		delegate void bufferDelegate(int[] data);

		List<int> rawDataRecieveBuffer = new List<int>();

		public void printRawData(int[] data) {
			if (rawDataBox.InvokeRequired) {
				try { rawDataBox.Invoke(new bufferDelegate(printRawData), new Object[] { data }); } catch { }
			} else {
				if (rawDataCheckBox.Checked) {
					lock (rawDataRecieveBuffer) {
						for (int i = 0; i < data.Length; i++) {

							if (data[i] == IR.RawDataLongSpace) {
								rawDataBox.AppendText(Environment.NewLine + Environment.NewLine);
								rawDataBox.AppendText("LONG");
								rawDataBox.AppendText(Environment.NewLine);
								rawDataBox.ScrollToCaret();
								rawDataCol = -1;

								rawDataRecieveBuffer.Clear();
							} else if (data[i] > 100000) {
								rawDataBox.AppendText(Environment.NewLine + Environment.NewLine);
								rawDataBox.AppendText(data[i].ToString().PadLeft(7) + ",");
								rawDataBox.AppendText(Environment.NewLine);
								rawDataBox.ScrollToCaret();
								rawDataCol = -1;

								rawDataRecieveBuffer.Add(data[i]);
							} else {
								rawDataBox.AppendText(data[i].ToString().PadLeft(7) + ",");

								rawDataRecieveBuffer.Add(data[i]);
							}

							rawDataCol++;
							if (rawDataCol >= 10) {
								rawDataBox.AppendText(Environment.NewLine);
								rawDataBox.ScrollToCaret();
								rawDataCol = 0;
							}
						}

						if (rawDataRecieveBuffer.Count > 0)
							sendRawDataBtn.Enabled = true;
						else
							sendRawDataBtn.Enabled = false;
					}
				}

			}
		}

		void ir_RawData(object sender, IRRawDataEventArgs e) {
			printRawData(e.Data);
		}

		void ir_Learn(object sender, IRLearnEventArgs e) {
			codeLearnTextBox.Text = "0x" + e.Code;

			bitLengthLearnTextBox.Text = e.CodeInfo.BitCount.ToString();
			lengthLearnTextBox.Text = e.CodeInfo.Length.GetDescription() + " Length";
			encodingTextBox.Text = e.CodeInfo.Encoding.GetDescription();
			gapTextBox.Text = e.CodeInfo.Gap.ToString();

			if (e.CodeInfo.Trail != 0)
				trailTextBox.Text = e.CodeInfo.Trail.ToString();
			else
				trailTextBox.Text = "None";

			oneTextBox.Text = e.CodeInfo.One[0].ToString() + ", " + e.CodeInfo.One[1].ToString();
			zeroTextBox.Text = e.CodeInfo.Zero[0].ToString() + ", " + e.CodeInfo.Zero[1].ToString();

			if (e.CodeInfo.Header[0] != 0)
				headerTextBox.Text = e.CodeInfo.Header[0].ToString() + ", " + e.CodeInfo.Header[1].ToString();
			else
				headerTextBox.Text = "No Header";

			if (!String.IsNullOrEmpty(e.CodeInfo.ToggleMask))
				toggleTextBox.Text = "0x" + e.CodeInfo.ToggleMask;
			else
				toggleTextBox.Text = "No Toggle Mask";

			if (e.CodeInfo.Repeat[0] != 0) {
				StringBuilder sb = new StringBuilder(e.CodeInfo.Repeat.Length * 2);
				foreach (int i in e.CodeInfo.Repeat) {
					if (i == 0)
						break;
					sb.Append(i.ToString());
					sb.Append(", ");
				}
				repeatTextBox.Text = sb.ToString();
			} else
				repeatTextBox.Text = "No Repeat Code";

			retransmitButton.Enabled = true;
		}

		void ir_Code(object sender, IRCodeEventArgs e) {
			codeTxt.Text = "0x" + e.Code;
			bitLengthTextBox.Text = e.BitCount.ToString();

			repeatCount++;
			if (!e.IsRepeat)
				repeatCount = 0;

			repeatCountTextBox.Text = repeatCount.ToString();
		}

		//attach event handler..populate the details fields as well as display the attached status.  enable the checkboxes to change
		//the values of the attributes of the IR reader such as enable or disable the antenna and onboard led.
		void ir_Attach(object sender, AttachEventArgs e) {
			IR attached = (IR)sender;
			//attachedTxt.Text = e.Device.Attached.ToString();
			phidgetInfoBox.FillPhidgetInfo((Phidget)sender);

			//clear fields
			codeTxt.Clear();
			repeatCountTextBox.Clear();
			bitLengthTextBox.Clear();
			codeBox.Visible = true;
			learnBox.Visible = true;
		}

		//detach event handler...clear all the fields, display the attached status, and disable the checkboxes.
		void ir_Detach(object sender, DetachEventArgs e) {
			IR detached = (IR)sender;
			phidgetInfoBox.Clear();
			cleaner.clean();

			retransmitButton.Enabled = false;

			codeBox.Visible = false;
			learnBox.Visible = false;

		}

		void ir_Error(object sender, ErrorEventArgs e) {
			errorBox.addMessage(e.Description);
		}

		private void retransmitThread() {
			Thread.Sleep(100);
			try {
				IRLearnedCode lastLearnedCode = ir.GetLastLearnedCode();
				IRCode lastCode = ir.GetLastCode();
				ir.Transmit(lastLearnedCode.Code, lastLearnedCode.CodeInfo);
				while (buttonPressed) {
					ir.TransmitRepeat();
				}
			} catch (PhidgetException) { }
		}

		private void retransmitButton_MouseDown(object sender, MouseEventArgs e) {
			buttonPressed = true;
			Thread buttonThread = new Thread(new ThreadStart(retransmitThread));
			buttonThread.Start();
		}

		private void retransmitButton_MouseUp(object sender, MouseEventArgs e) {
			buttonPressed = false;
		}

		private void rawDataCheckBox_CheckedChanged(object sender, EventArgs e) {
			if (rawDataCheckBox.Checked) {
				rawDataBox.Visible = true;
				clearButton.Visible = true;
				sendRawDataBtn.Visible = true;
			} else {
				rawDataBox.Visible = false;
				clearButton.Visible = false;
				sendRawDataBtn.Visible = false;
			}
		}

		private void clearButton_Click(object sender, EventArgs e) {
			rawDataBox.Clear();
			lock (rawDataRecieveBuffer) {
				rawDataRecieveBuffer.Clear();
			}
			sendRawDataBtn.Enabled = false;
		}

		//example of sending RAW Data
		private void button1_Click(object sender, EventArgs e) {
			String buttonTxt = sendRawDataBtn.Text;
			try {
				// We can send a max of 199 array items. We're going to look for 'gaps' in the captured data stream
				// and split up the sends around these. We'll choose 50 ms as a good gap time threshold.
				// A single button press won't have any gaps - these are seen when a button is held down.
				List<rawDataChunk> rawChunks = new List<rawDataChunk>();
				lock (rawDataRecieveBuffer) {
					rawDataChunk chunk = new rawDataChunk();
					foreach (int blip in rawDataRecieveBuffer) {
						if (blip >= 50000) {
							chunk.gap = blip;
							rawChunks.Add(chunk);
							chunk = new rawDataChunk();
						} else {
							chunk.datalist.Add(blip);
						}
					}
					chunk.gap = 150000; //laset gap, nice and big.
					rawChunks.Add(chunk);
				}

				sendRawDataBtn.Text = "Sending Data...";
				Application.DoEvents();

				foreach (rawDataChunk chunk in rawChunks)
					ir.TransmitRaw(chunk.data, 0, 0, chunk.gap);
			} catch (PhidgetException ex) {
				errorBox.addMessage("Error sending raw data: " + ex.Message);
			} finally {
				sendRawDataBtn.Text = buttonTxt;
			}
		}

		private void sendTestCodeBtn_Click(object sender, EventArgs e) {

			IRCodeInfo codeInfo = new IRCodeInfo {
				BitCount = 32,
				Encoding = IRCodeEncoding.Space,
				Zero = new int[] { 580, 580 },
				One = new int[] { 580, 1710 },
				Header = new int[] { 9080, 4550 },
				Trail = 580,
				Gap = 108500,
				Repeat = new int[] { 1000, 1500, 2000, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
				//MinRepeat = 2,
				//ToggleMask = "00000001"
			};

			// Transmit enough for another PhidgetIR to get a learn event
			ir.Transmit("deadbeef", codeInfo);
			ir.TransmitRepeat();
			ir.TransmitRepeat();
			ir.TransmitRepeat();
			ir.TransmitRepeat();
			ir.TransmitRepeat();
		}
	}

	class rawDataChunk {
		public List<int> datalist = new List<int>();
		public int[] data {
			get {
				return datalist.ToArray();
			}
		}
		public int gap;


	}
}